package com.icontrols.kakao.common;

import java.util.Arrays;

import org.slf4j.LoggerFactory;

import com.icontrols.kakao.common.utils.BeanUtils;
import com.icontrols.kakao.repo.ThingsInfoRepository;
import com.icontrols.kakao.socket.wallpad.KakaoMsgHandler;

import ch.qos.logback.classic.Logger;

public final class THINGS_INFO {
	
	public static final int THINGS_LIGHT 		= 1,  THINGS_TEMPER  	= 2,  THINGS_ELECPLUG 	= 3,  THINGS_GAS 		= 4,
							THINGS_ILGALSWITCH 	= 5,  THINGS_WALLPAD  	= 6,  THINGS_VENTIL 	= 7,  THINGS_NOTICE 	= 8,
							THINGS_VOTE			= 9,  THINGS_PARKING  	= 10, THINGS_PARCEL 	= 11, THINGS_HEMS 		= 12,
							THINGS_CARIN 		= 13, THINGS_ELEVATOR 	= 14, THINGS_VISITOR 	= 15, THINGS_EMERGENCY 	= 16,
							THINGS_LIGHT_DIMMER = 17, THINGS_COLOR_LIGHT_DIMMER = 18;
	
	public enum THINGS_CAPABILITY{
		LIVINGLIGHT("livingLight","{\"capabilities\" : [ { \"name\" : \"Power\" } ]}"),
		LIGHT("light","{\"capabilities\" : [ { \"name\" : \"Power\" } ]}"),
		LIGHT_DIMMER("dimming","{\"capabilities\" : [ { \"name\" : \"Power\" }, { \"name\" : \"Brightness\" } ]}"),
		COLOR_LIGHT("colorLight", "{\"capabilities\" : [ { \"name\" : \"Power\" }, { \"name\" : \"ColorMode\"}, { \"name\" : \"ColorTemperature\"} ]}"),
		COLOR_LIGHT_DIMMER("colorDimming", "{\"capabilities\" : [ { \"name\" : \"Power\" }, { \"name\" : \"Brightness\" }, { \"name\" : \"ColorMode\"}, { \"name\" : \"ColorTemperature\", \"attributes\" : { \"colorTemperature\": { \"min\":3000, \"max\":5700 } } } ]}"),
		TEMPER("temper", "{\"capabilities\" : [ { \"name\" : \"Power\" }, { \"name\" : \"Temperature\" }, { \"name\" : \"TemperatureSetpoint\" } ]}"),
		VENTIL("ventil","{\"capabilities\" : [ { \"name\" : \"Power\" }, { \"name\" : \"AirFlow\" } ]}"),
		GAS("gas","{\"capabilities\" : [ { \"name\" : \"Lock\" } ] }"),
		ELEVATOR("elevator", "{\"capabilities\" : [ { \"name\" : \"ElevatorCall\" } ] }"),
		ELECTRIC("electric", "{\"capabilities\" : [ { \"name\" : \"Power\" } ]}");
		
		private String thingsName;
		private String capJsonStr;
		
		private THINGS_CAPABILITY(String thingsName, String capJsonStr) {
			this.thingsName = thingsName;
			this.capJsonStr = capJsonStr;
		}
		public String getThingsName() {
			return thingsName;
		}
		public void setThingsName(String thingsName) {
			this.thingsName = thingsName;
		}
		public String getCapJsonStr() {
			return capJsonStr;
		}
		public void setCapJsonStr(String capJsonStr) {
			this.capJsonStr = capJsonStr;
		}
	}
	
	public enum DEVICE_DETAIL{
		LIVINGLIGHT					("livinglight", 				"Light", 			"거실조명"),
		LIGHT01						("light01",						"Light",			"주방조명"),
		LIGHT02						("light02",						"Light",			"방1조명"),
		LIGHT03						("light03",						"Light",			"방2조명"),
		LIGHT04						("light04",						"Light",			"방3조명"),
		DIMMER_LIVINGLIGHT			("dimmer_livinglight", 			"LightDimmer", 		"거실조명"),
		DIMMER_LIGHT01				("dimmer_light01",				"LightDimmer",		"주방조명"),
		DIMMER_LIGHT02				("dimmer_light02",				"LightDimmer",		"방1조명"),
		DIMMER_LIGHT03				("dimmer_light03",				"LightDimmer",		"방2조명"),
		DIMMER_LIGHT04				("dimmer_light04",				"LightDimmer",		"방3조명"),
		COLOR_DIMMER_LIVINGLIGHT	("colorDimmer_livinglight", 	"ColorLightDimmer",	"거실조명"),
		COLOR_DIMMER_LIGHT01		("colorDimmer_light01",			"ColorLightDimmer",	"주방조명"),
		COLOR_DIMMER_LIGHT02		("colorDimmer_light02",			"ColorLightDimmer",	"방1조명"),
		COLOR_DIMMER_LIGHT03		("colorDimmer_light03",			"ColorLightDimmer",	"방2조명"),
		COLOR_DIMMER_LIGHT04		("colorDimmer_light04",			"ColorLightDimmer",	"방3조명"),
		VENTIL						("ventil",						"AirVentilator",	"환기"),
		GAS							("gas",							"GasValve",			"가스밸브"),
		ELEVATOR					("elevator",					"ElevatorCaller",	"엘리베이터"),
		TEMPER1						("temper_1",					"FloorHeater",		"거실난방"),
		TEMPER2						("temper_2",					"FloorHeater",		"주방난방"),
		TEMPER3						("temper_3",					"FloorHeater",		"방1난방"),
		TEMPER4						("temper_4",					"FloorHeater",		"방2난방"),
		TEMPER5						("temper_5",					"FloorHeater",		"방3난방"),
		ELECTRIC01					("electric01",					"PowerOutlet",		"거실콘센트"),
		ELECTRIC02					("electric02",					"PowerOutlet",		"주방콘센트"),
		ELECTRIC03					("electric03",					"PowerOutlet",		"방1콘센트"),
		ELECTRIC04					("electric04",					"PowerOutlet",		"방2콘센트"),
		ELECTRIC05					("electric05",					"PowerOutlet",		"방3콘센트")
		;
		
		
		private String modelId;
		
		private String displayType;
		
		private String displayName;

		
		private DEVICE_DETAIL(String modelId, String displayType, String displayName) {
			this.modelId = modelId;
			this.displayType = displayType;
			this.displayName = displayName;
		}

		public String getModelId() {
			return modelId;
		}

		public void setModelId(String modelId) {
			this.modelId = modelId;
		}

		public String getDisplayType() {
			return displayType;
		}

		public void setDisplayType(String displayType) {
			this.displayType = displayType;
		}

		public String getDisplayName() {
			return displayName;
		}

		public void setDisplayName(String displayName) {
			this.displayName = displayName;
		}
	}
	
	public enum THINGS{
		//endpoint는 카카오 API와 연동될 기기 endpoint
		// name 은 우리쪽에서 정의된 기기 명
		// desc 는 기기 설명
		LIGHT			(1,	"LIGHT", 			"Light", 				"조명스위치"),				
		TEMPER			(2, "TEMPER", 			"FloorHeater", 			"난방"),
		ELECPLUG		(3, "ELECTRIC", 		"PowerOutlet", 			"전기"),
		GAS				(4, "GAS", 				"GasValve", 			"가스밸브"),
		ILGALSWITCH		(5, "ILGALSWITCH", 		"ilgal", 				"일괄조명스위치"),
		WALLPAD			(6, "WALLPAD", 			"walpad", 				"월패드"),
		VENTIL			(7, "VENTIL", 			"AirVentilator", 		"환기시스템"),
		NOTICE			(8, "NOTICE", 			"InformationReporter", 	"공지사항"),
		VOTE			(9, "VOTE", 			"vote", 				"주민투표"),
		PARKING			(10, "PARKING", 		"parking", 				"주차위치"),
		PARCEL			(11, "PARCEL", 			"parcel", 				"택배도착"),
		HEMS			(12, "HEMS", 			"hems", 				"원격검침"),
		CARIN			(13, "CARIN", 			"carin", 				"주차관제(입차/출차)"),
		ELEVATOR		(14, "elevator",		"ElevatorCaller", 		"엘리베이터"),
		VISITOR			(15, "VISITOR", 		"visitor", 				"방문자"),
		EMERGENCY		(16, "EMERGENCY", 		"emergency", 			"방범"),
		LIGHTDIMMER		(17, "DIMMER", 	"LightDimmer", 			"디밍제어 조명"),
		COLORLIGHTDIMMER(18, "COLORDIMMER", "ColorLightDimmer", 	"색온도 디밍제어 조명")
		;
		
		private int endPoint;
		private String thingsName;
		private String kakaoDisplayName;
		private String desc;
		
		private THINGS(int endPoint, String thingsName, String kakaoDisplayName, String desc) {
			this.endPoint = endPoint;
			this.thingsName = thingsName;
			this.kakaoDisplayName = kakaoDisplayName;
			this.desc = desc;
		}

		public int getEndPoint() {
			return endPoint;
		}

		public void setEndPoint(int endPoint) {
			this.endPoint = endPoint;
		}

		public String getThingsName() {
			return thingsName;
		}

		public void setThingsName(String thingsName) {
			this.thingsName = thingsName;
		}

		public String getKakaoDisplayName() {
			return kakaoDisplayName;
		}

		public void setKakaoDisplayName(String kakaoDisplayName) {
			this.kakaoDisplayName = kakaoDisplayName;
		}

		public String getDesc() {
			return desc;
		}

		public void setDesc(String desc) {
			this.desc = desc;
		}
	}
	
	public static boolean isExistThings(String modelId) {
		THINGS_INFO.THINGS things = getThingsType(modelId);
		
		return things == null ? false:true;
	}     
	
	public static THINGS_INFO.DEVICE_DETAIL getDeviceInfo(String modelId, String status){
		THINGS_INFO.DEVICE_DETAIL[] list = THINGS_INFO.DEVICE_DETAIL.values();
		
		for(THINGS_INFO.DEVICE_DETAIL deviceDetail : list) {
			if(deviceDetail.getModelId().equals(modelId)) {
				return deviceDetail;
			}
		}
		return null; 
	}
	
	public static THINGS_INFO.THINGS getThingsType(String modelId){    
		ThingsInfoRepository repo = (ThingsInfoRepository)BeanUtils.getBean("thingsInfoRepository");
		
		THINGS_INFO.THINGS[] list = THINGS_INFO.THINGS.values();
		
		String[] splitModelId = modelId.split("_");
		
		for(THINGS_INFO.THINGS things:list) {
//			if(modelId.toLowerCase().contains(things.getThingsName().toLowerCase())) {
//				return things;
//			}
			if(splitModelId[0].equalsIgnoreCase(things.getThingsName()) || splitModelId[0].contains(things.getThingsName().toLowerCase())) {
				System.out.println(splitModelId[0] + "--" + things.getThingsName());
				return things;
			}
		}
		System.out.println("[ERROR] - " + modelId + " --> NULL");
		return null;
	}
	
	public static boolean isAvailableThings(String modelId) {
		return Arrays.asList("livinglight", "light", "temper", "ventil", "gas", "electric").contains(modelId);
	}
	
}
